@@ -11,41 +11,49 @@
 #'
 #' nodes <- data.frame(id = 1:10, label = paste("Label", 1:10), 
 #'  group = sample(c("A", "B"), 10, replace = TRUE))
-#' edges <- data.frame(from = c(2,5,10), to = c(1,2,10))
+#'  edges <- data.frame(from = c(2,5,10), to = c(1,2,10))
 #'
-#' visNetwork(nodes, edges, legend = TRUE) %>%
+#' visNetwork(nodes, edges) %>%
+#'  visLegend() %>%
 #'  visGroups(groupname = "A", color = "red", shape = "database") %>%
 #'  visGroups(groupname = "B", color = "yellow", shape = "triangle")
 #'  
 #'@seealso \link{visNodes} for nodes options, \link{visEdges} for edges options, \link{visGroups} for groups options, 
 #'\link{visLegend} for adding legend, \link{visOptions} for custom option, \link{visLayout} & \link{visHierarchicalLayout} for layout, 
-#'\link{visPhysics} for control physics, \link{visInteraction} for interaction, \link{visDocumentation}, \link{visEvents}, \link{visConfigure} ...
-#'
+#'\link{visPhysics} for control physics, \link{visInteraction} for interaction, \link{visNetworkProxy} & \link{visFocus} & \link{visFit} for animation within shiny,
+#'\link{visDocumentation}, \link{visEvents}, \link{visConfigure} ...
 #'
 #' @export
 #'
 
 visGroups <- function(graph,
-                      useDefaultGroups = NULL,
+                      useDefaultGroups = TRUE,
                       groupname = NULL,
                       ...){
 
-  groups <- list()
-  groups$useDefaultGroups = useDefaultGroups
-  graph$x$options$groups <- mergeLists(graph$x$options$groups, groups)
+  if(!any(class(graph) %in% c("visNetwork", "visNetwork_Proxy"))){
+    stop("graph must be a visNetwork or a visNetworkProxy object")
+  }
   
+  groups <- list()
+  groups$useDefaultGroups <- useDefaultGroups
+
   params <- list(...)
   
   if(length(params) > 0){
     if(is.null(groupname)){
       stop("Must have a groupname to identify group")
     }
-    
-    groups <- list(list(...))
-    names(groups) <- groupname
-    
-    graph$x$options$groups <- mergeLists(graph$x$options$groups, groups)
+    tmp <- list(...)
+    groups[[groupname]] <- tmp
   }
   
+  if(any(class(graph) %in% "visNetwork_Proxy")){
+    options <- list(groups = groups)
+    data <- list(id = graph$id, options = options)
+    graph$session$sendCustomMessage("Options",data)
+  }else{
+    graph$x$options$groups <- mergeLists(graph$x$options$groups, groups)
+  }
   graph
 }
