@@ -47,8 +47,8 @@
 #'
 #'@seealso \link{visNodes} for nodes options, \link{visEdges} for edges options, \link{visGroups} for groups options, 
 #'\link{visLegend} for adding legend, \link{visOptions} for custom option, \link{visLayout} & \link{visHierarchicalLayout} for layout, 
-#'\link{visPhysics} for control physics, \link{visInteraction} for interaction, \link{visDocumentation}, \link{visEvents}, \link{visConfigure} ...
-#'
+#'\link{visPhysics} for control physics, \link{visInteraction} for interaction, \link{visNetworkProxy} & \link{visFocus} & \link{visFit} for animation within shiny,
+#'\link{visDocumentation}, \link{visEvents}, \link{visConfigure} ...
 #'
 #' @export
 visEvents <- function(graph,
@@ -82,6 +82,14 @@ visEvents <- function(graph,
                       afterDrawing = NULL,
                       animationFinished = NULL){
 
+  if(any(class(graph) %in% "visNetwork_Proxy")){
+    stop("Can't use visEvents with visNetworkProxy object")
+  }
+  
+  if(!any(class(graph) %in% "visNetwork")){
+    stop("graph must be a visNetwork object")
+  }
+  
   events <- list()
   events$click  <- JS(click)
   events$doubleClick  <- JS(doubleClick)
